#ifndef __ENCODE_DRAW_H__
#define __ENCODE_DRAW_H__

#include "SunIM.h"
#include "koinput.h"
#include "encode.h"
extern int encode_to_unicode(int encode, unsigned char *from, int from_len, UTFCHAR *to, int to_len);
extern void encode_draw_preedit(iml_session_t *s, int encode, unsigned char *preedit_buf, int caret);
extern void encode_draw_status(iml_session_t *s, int encode, unsigned char *status_buf);
extern void encode_draw_candidates(iml_session_t *s, int encode, unsigned char **lookup_buf, int lookup_num, int lookup_label_type);
extern void encode_commit(iml_session_t *s, int encode, unsigned char *commit_buf);
#endif /* __ENCODE_DRAW_H__ */
